# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/orderer/configuration.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/orderer/configuration.proto',
  package='orderer',
  syntax='proto3',
  serialized_options=_b('\n%org.hyperledger.fabric.protos.ordererZ,github.com/hyperledger/fabric/protos/orderer'),
  serialized_pb=_b('\n&hfc/protos/orderer/configuration.proto\x12\x07orderer\"/\n\rConsensusType\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x10\n\x08metadata\x18\x02 \x01(\x0c\"_\n\tBatchSize\x12\x19\n\x11max_message_count\x18\x01 \x01(\r\x12\x1a\n\x12\x61\x62solute_max_bytes\x18\x02 \x01(\r\x12\x1b\n\x13preferred_max_bytes\x18\x03 \x01(\r\"\x1f\n\x0c\x42\x61tchTimeout\x12\x0f\n\x07timeout\x18\x01 \x01(\t\"\x1f\n\x0cKafkaBrokers\x12\x0f\n\x07\x62rokers\x18\x01 \x03(\t\"(\n\x13\x43hannelRestrictions\x12\x11\n\tmax_count\x18\x01 \x01(\x04\x42U\n%org.hyperledger.fabric.protos.ordererZ,github.com/hyperledger/fabric/protos/ordererb\x06proto3')
)




_CONSENSUSTYPE = _descriptor.Descriptor(
  name='ConsensusType',
  full_name='orderer.ConsensusType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='orderer.ConsensusType.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='orderer.ConsensusType.metadata', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=51,
  serialized_end=98,
)


_BATCHSIZE = _descriptor.Descriptor(
  name='BatchSize',
  full_name='orderer.BatchSize',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='max_message_count', full_name='orderer.BatchSize.max_message_count', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='absolute_max_bytes', full_name='orderer.BatchSize.absolute_max_bytes', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='preferred_max_bytes', full_name='orderer.BatchSize.preferred_max_bytes', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=100,
  serialized_end=195,
)


_BATCHTIMEOUT = _descriptor.Descriptor(
  name='BatchTimeout',
  full_name='orderer.BatchTimeout',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timeout', full_name='orderer.BatchTimeout.timeout', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=197,
  serialized_end=228,
)


_KAFKABROKERS = _descriptor.Descriptor(
  name='KafkaBrokers',
  full_name='orderer.KafkaBrokers',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='brokers', full_name='orderer.KafkaBrokers.brokers', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=230,
  serialized_end=261,
)


_CHANNELRESTRICTIONS = _descriptor.Descriptor(
  name='ChannelRestrictions',
  full_name='orderer.ChannelRestrictions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='max_count', full_name='orderer.ChannelRestrictions.max_count', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=263,
  serialized_end=303,
)

DESCRIPTOR.message_types_by_name['ConsensusType'] = _CONSENSUSTYPE
DESCRIPTOR.message_types_by_name['BatchSize'] = _BATCHSIZE
DESCRIPTOR.message_types_by_name['BatchTimeout'] = _BATCHTIMEOUT
DESCRIPTOR.message_types_by_name['KafkaBrokers'] = _KAFKABROKERS
DESCRIPTOR.message_types_by_name['ChannelRestrictions'] = _CHANNELRESTRICTIONS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ConsensusType = _reflection.GeneratedProtocolMessageType('ConsensusType', (_message.Message,), dict(
  DESCRIPTOR = _CONSENSUSTYPE,
  __module__ = 'hfc.protos.orderer.configuration_pb2'
  # @@protoc_insertion_point(class_scope:orderer.ConsensusType)
  ))
_sym_db.RegisterMessage(ConsensusType)

BatchSize = _reflection.GeneratedProtocolMessageType('BatchSize', (_message.Message,), dict(
  DESCRIPTOR = _BATCHSIZE,
  __module__ = 'hfc.protos.orderer.configuration_pb2'
  # @@protoc_insertion_point(class_scope:orderer.BatchSize)
  ))
_sym_db.RegisterMessage(BatchSize)

BatchTimeout = _reflection.GeneratedProtocolMessageType('BatchTimeout', (_message.Message,), dict(
  DESCRIPTOR = _BATCHTIMEOUT,
  __module__ = 'hfc.protos.orderer.configuration_pb2'
  # @@protoc_insertion_point(class_scope:orderer.BatchTimeout)
  ))
_sym_db.RegisterMessage(BatchTimeout)

KafkaBrokers = _reflection.GeneratedProtocolMessageType('KafkaBrokers', (_message.Message,), dict(
  DESCRIPTOR = _KAFKABROKERS,
  __module__ = 'hfc.protos.orderer.configuration_pb2'
  # @@protoc_insertion_point(class_scope:orderer.KafkaBrokers)
  ))
_sym_db.RegisterMessage(KafkaBrokers)

ChannelRestrictions = _reflection.GeneratedProtocolMessageType('ChannelRestrictions', (_message.Message,), dict(
  DESCRIPTOR = _CHANNELRESTRICTIONS,
  __module__ = 'hfc.protos.orderer.configuration_pb2'
  # @@protoc_insertion_point(class_scope:orderer.ChannelRestrictions)
  ))
_sym_db.RegisterMessage(ChannelRestrictions)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from hfc.protos.orderer import cluster_pb2 as hfc_dot_protos_dot_orderer_dot_cluster__pb2


class ClusterStub(object):
  """Cluster defines communication between cluster members.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Submit = channel.stream_stream(
        '/orderer.Cluster/Submit',
        request_serializer=hfc_dot_protos_dot_orderer_dot_cluster__pb2.SubmitRequest.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_orderer_dot_cluster__pb2.SubmitResponse.FromString,
        )
    self.Step = channel.unary_unary(
        '/orderer.Cluster/Step',
        request_serializer=hfc_dot_protos_dot_orderer_dot_cluster__pb2.StepRequest.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_orderer_dot_cluster__pb2.StepResponse.FromString,
        )


class ClusterServicer(object):
  """Cluster defines communication between cluster members.
  """

  def Submit(self, request_iterator, context):
    """Submit submits transactions to a cluster member
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Step(self, request, context):
    """Step passes an implementation-specific message to another cluster member.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_ClusterServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Submit': grpc.stream_stream_rpc_method_handler(
          servicer.Submit,
          request_deserializer=hfc_dot_protos_dot_orderer_dot_cluster__pb2.SubmitRequest.FromString,
          response_serializer=hfc_dot_protos_dot_orderer_dot_cluster__pb2.SubmitResponse.SerializeToString,
      ),
      'Step': grpc.unary_unary_rpc_method_handler(
          servicer.Step,
          request_deserializer=hfc_dot_protos_dot_orderer_dot_cluster__pb2.StepRequest.FromString,
          response_serializer=hfc_dot_protos_dot_orderer_dot_cluster__pb2.StepResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'orderer.Cluster', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/peer/peer.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hfc.protos.peer import proposal_pb2 as hfc_dot_protos_dot_peer_dot_proposal__pb2
from hfc.protos.peer import proposal_response_pb2 as hfc_dot_protos_dot_peer_dot_proposal__response__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/peer/peer.proto',
  package='protos',
  syntax='proto3',
  serialized_options=_b('\n\"org.hyperledger.fabric.protos.peerZ)github.com/hyperledger/fabric/protos/peer'),
  serialized_pb=_b('\n\x1ahfc/protos/peer/peer.proto\x12\x06protos\x1a\x1ehfc/protos/peer/proposal.proto\x1a\'hfc/protos/peer/proposal_response.proto\"\x16\n\x06PeerID\x12\x0c\n\x04name\x18\x01 \x01(\t\";\n\x0cPeerEndpoint\x12\x1a\n\x02id\x18\x01 \x01(\x0b\x32\x0e.protos.PeerID\x12\x0f\n\x07\x61\x64\x64ress\x18\x02 \x01(\t2Q\n\x08\x45ndorser\x12\x45\n\x0fProcessProposal\x12\x16.protos.SignedProposal\x1a\x18.protos.ProposalResponse\"\x00\x42O\n\"org.hyperledger.fabric.protos.peerZ)github.com/hyperledger/fabric/protos/peerb\x06proto3')
  ,
  dependencies=[hfc_dot_protos_dot_peer_dot_proposal__pb2.DESCRIPTOR,hfc_dot_protos_dot_peer_dot_proposal__response__pb2.DESCRIPTOR,])




_PEERID = _descriptor.Descriptor(
  name='PeerID',
  full_name='protos.PeerID',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='protos.PeerID.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=111,
  serialized_end=133,
)


_PEERENDPOINT = _descriptor.Descriptor(
  name='PeerEndpoint',
  full_name='protos.PeerEndpoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='protos.PeerEndpoint.id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='address', full_name='protos.PeerEndpoint.address', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=135,
  serialized_end=194,
)

_PEERENDPOINT.fields_by_name['id'].message_type = _PEERID
DESCRIPTOR.message_types_by_name['PeerID'] = _PEERID
DESCRIPTOR.message_types_by_name['PeerEndpoint'] = _PEERENDPOINT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PeerID = _reflection.GeneratedProtocolMessageType('PeerID', (_message.Message,), dict(
  DESCRIPTOR = _PEERID,
  __module__ = 'hfc.protos.peer.peer_pb2'
  # @@protoc_insertion_point(class_scope:protos.PeerID)
  ))
_sym_db.RegisterMessage(PeerID)

PeerEndpoint = _reflection.GeneratedProtocolMessageType('PeerEndpoint', (_message.Message,), dict(
  DESCRIPTOR = _PEERENDPOINT,
  __module__ = 'hfc.protos.peer.peer_pb2'
  # @@protoc_insertion_point(class_scope:protos.PeerEndpoint)
  ))
_sym_db.RegisterMessage(PeerEndpoint)


DESCRIPTOR._options = None

_ENDORSER = _descriptor.ServiceDescriptor(
  name='Endorser',
  full_name='protos.Endorser',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=196,
  serialized_end=277,
  methods=[
  _descriptor.MethodDescriptor(
    name='ProcessProposal',
    full_name='protos.Endorser.ProcessProposal',
    index=0,
    containing_service=None,
    input_type=hfc_dot_protos_dot_peer_dot_proposal__pb2._SIGNEDPROPOSAL,
    output_type=hfc_dot_protos_dot_peer_dot_proposal__response__pb2._PROPOSALRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_ENDORSER)

DESCRIPTOR.services_by_name['Endorser'] = _ENDORSER

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from hfc.protos.common import common_pb2 as hfc_dot_protos_dot_common_dot_common__pb2
from hfc.protos.orderer import ab_pb2 as hfc_dot_protos_dot_orderer_dot_ab__pb2


class AtomicBroadcastStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Broadcast = channel.stream_stream(
        '/orderer.AtomicBroadcast/Broadcast',
        request_serializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_orderer_dot_ab__pb2.BroadcastResponse.FromString,
        )
    self.Deliver = channel.stream_stream(
        '/orderer.AtomicBroadcast/Deliver',
        request_serializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_orderer_dot_ab__pb2.DeliverResponse.FromString,
        )


class AtomicBroadcastServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def Broadcast(self, request_iterator, context):
    """broadcast receives a reply of Acknowledgement for each common.Envelope in order, indicating success or type of failure
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Deliver(self, request_iterator, context):
    """deliver first requires an Envelope of type DELIVER_SEEK_INFO with Payload data as a mashaled SeekInfo message, then a stream of block replies is received.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_AtomicBroadcastServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Broadcast': grpc.stream_stream_rpc_method_handler(
          servicer.Broadcast,
          request_deserializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.FromString,
          response_serializer=hfc_dot_protos_dot_orderer_dot_ab__pb2.BroadcastResponse.SerializeToString,
      ),
      'Deliver': grpc.stream_stream_rpc_method_handler(
          servicer.Deliver,
          request_deserializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.FromString,
          response_serializer=hfc_dot_protos_dot_orderer_dot_ab__pb2.DeliverResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'orderer.AtomicBroadcast', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import unittest
from shutil import rmtree

from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.serialization import Encoding

from hfc.fabric.user import User, Enrollment
from hfc.util.crypto.crypto import ecies
from hfc.util.keyvaluestore import file_key_value_store


class UserTest(unittest.TestCase):
    def setUp(self):
        self.base_path = '/tmp/fabric-sdk-py'
        self.path = os.path.join(self.base_path, 'user-state-store')

    def tearDown(self):
        rmtree(self.base_path)

    def test_create_user(self):
        store = file_key_value_store(self.path)
        user = User('test_user', 'peerOrg1', store)
        self.assertTrue(isinstance(user, User))

    def test_user_state(self):
        store = file_key_value_store(self.path)
        user = User('test_user', 'peerOrg1', store)
        user.roles = ['test']

        ec = ecies()

        enrollment = Enrollment(ec.generate_private_key(), "dasdasdasdasdasd")
        user.enrollment = enrollment

        user1 = User('test_user', 'peerOrg1', store)
        self.assertTrue(user1.roles == ['test'])
        self.assertTrue(user1.enrollment.cert == "dasdasdasdasdasd")
        pub_key = user1.enrollment.private_key.public_key()
        self.assertTrue(pub_key.public_bytes(
            encoding=Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo)
            .startswith(b'-----BEGIN PUBLIC KEY'))


if __name__ == '__main__':
    unittest.main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/peer/events.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hfc.protos.common import common_pb2 as hfc_dot_protos_dot_common_dot_common__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from hfc.protos.peer import chaincode_event_pb2 as hfc_dot_protos_dot_peer_dot_chaincode__event__pb2
from hfc.protos.peer import transaction_pb2 as hfc_dot_protos_dot_peer_dot_transaction__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/peer/events.proto',
  package='protos',
  syntax='proto3',
  serialized_options=_b('\n\"org.hyperledger.fabric.protos.peerB\rEventsPackageZ)github.com/hyperledger/fabric/protos/peer'),
  serialized_pb=_b('\n\x1chfc/protos/peer/events.proto\x12\x06protos\x1a\x1ehfc/protos/common/common.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a%hfc/protos/peer/chaincode_event.proto\x1a!hfc/protos/peer/transaction.proto\"o\n\rFilteredBlock\x12\x12\n\nchannel_id\x18\x01 \x01(\t\x12\x0e\n\x06number\x18\x02 \x01(\x04\x12:\n\x15\x66iltered_transactions\x18\x04 \x03(\x0b\x32\x1b.protos.FilteredTransaction\"\xc6\x01\n\x13\x46ilteredTransaction\x12\x0c\n\x04txid\x18\x01 \x01(\t\x12 \n\x04type\x18\x02 \x01(\x0e\x32\x12.common.HeaderType\x12\x34\n\x12tx_validation_code\x18\x03 \x01(\x0e\x32\x18.protos.TxValidationCode\x12\x41\n\x13transaction_actions\x18\x04 \x01(\x0b\x32\".protos.FilteredTransactionActionsH\x00\x42\x06\n\x04\x44\x61ta\"X\n\x1a\x46ilteredTransactionActions\x12:\n\x11\x63haincode_actions\x18\x01 \x03(\x0b\x32\x1f.protos.FilteredChaincodeAction\"J\n\x17\x46ilteredChaincodeAction\x12/\n\x0f\x63haincode_event\x18\x01 \x01(\x0b\x32\x16.protos.ChaincodeEvent\"\x8c\x01\n\x0f\x44\x65liverResponse\x12 \n\x06status\x18\x01 \x01(\x0e\x32\x0e.common.StatusH\x00\x12\x1e\n\x05\x62lock\x18\x02 \x01(\x0b\x32\r.common.BlockH\x00\x12/\n\x0e\x66iltered_block\x18\x03 \x01(\x0b\x32\x15.protos.FilteredBlockH\x00\x42\x06\n\x04Type2\x89\x01\n\x07\x44\x65liver\x12:\n\x07\x44\x65liver\x12\x10.common.Envelope\x1a\x17.protos.DeliverResponse\"\x00(\x01\x30\x01\x12\x42\n\x0f\x44\x65liverFiltered\x12\x10.common.Envelope\x1a\x17.protos.DeliverResponse\"\x00(\x01\x30\x01\x42^\n\"org.hyperledger.fabric.protos.peerB\rEventsPackageZ)github.com/hyperledger/fabric/protos/peerb\x06proto3')
  ,
  dependencies=[hfc_dot_protos_dot_common_dot_common__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,hfc_dot_protos_dot_peer_dot_chaincode__event__pb2.DESCRIPTOR,hfc_dot_protos_dot_peer_dot_transaction__pb2.DESCRIPTOR,])




_FILTEREDBLOCK = _descriptor.Descriptor(
  name='FilteredBlock',
  full_name='protos.FilteredBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channel_id', full_name='protos.FilteredBlock.channel_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='number', full_name='protos.FilteredBlock.number', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filtered_transactions', full_name='protos.FilteredBlock.filtered_transactions', index=2,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=179,
  serialized_end=290,
)


_FILTEREDTRANSACTION = _descriptor.Descriptor(
  name='FilteredTransaction',
  full_name='protos.FilteredTransaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='txid', full_name='protos.FilteredTransaction.txid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='protos.FilteredTransaction.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tx_validation_code', full_name='protos.FilteredTransaction.tx_validation_code', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='transaction_actions', full_name='protos.FilteredTransaction.transaction_actions', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Data', full_name='protos.FilteredTransaction.Data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=293,
  serialized_end=491,
)


_FILTEREDTRANSACTIONACTIONS = _descriptor.Descriptor(
  name='FilteredTransactionActions',
  full_name='protos.FilteredTransactionActions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chaincode_actions', full_name='protos.FilteredTransactionActions.chaincode_actions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=493,
  serialized_end=581,
)


_FILTEREDCHAINCODEACTION = _descriptor.Descriptor(
  name='FilteredChaincodeAction',
  full_name='protos.FilteredChaincodeAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chaincode_event', full_name='protos.FilteredChaincodeAction.chaincode_event', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=583,
  serialized_end=657,
)


_DELIVERRESPONSE = _descriptor.Descriptor(
  name='DeliverResponse',
  full_name='protos.DeliverResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='protos.DeliverResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='block', full_name='protos.DeliverResponse.block', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filtered_block', full_name='protos.DeliverResponse.filtered_block', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Type', full_name='protos.DeliverResponse.Type',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=660,
  serialized_end=800,
)

_FILTEREDBLOCK.fields_by_name['filtered_transactions'].message_type = _FILTEREDTRANSACTION
_FILTEREDTRANSACTION.fields_by_name['type'].enum_type = hfc_dot_protos_dot_common_dot_common__pb2._HEADERTYPE
_FILTEREDTRANSACTION.fields_by_name['tx_validation_code'].enum_type = hfc_dot_protos_dot_peer_dot_transaction__pb2._TXVALIDATIONCODE
_FILTEREDTRANSACTION.fields_by_name['transaction_actions'].message_type = _FILTEREDTRANSACTIONACTIONS
_FILTEREDTRANSACTION.oneofs_by_name['Data'].fields.append(
  _FILTEREDTRANSACTION.fields_by_name['transaction_actions'])
_FILTEREDTRANSACTION.fields_by_name['transaction_actions'].containing_oneof = _FILTEREDTRANSACTION.oneofs_by_name['Data']
_FILTEREDTRANSACTIONACTIONS.fields_by_name['chaincode_actions'].message_type = _FILTEREDCHAINCODEACTION
_FILTEREDCHAINCODEACTION.fields_by_name['chaincode_event'].message_type = hfc_dot_protos_dot_peer_dot_chaincode__event__pb2._CHAINCODEEVENT
_DELIVERRESPONSE.fields_by_name['status'].enum_type = hfc_dot_protos_dot_common_dot_common__pb2._STATUS
_DELIVERRESPONSE.fields_by_name['block'].message_type = hfc_dot_protos_dot_common_dot_common__pb2._BLOCK
_DELIVERRESPONSE.fields_by_name['filtered_block'].message_type = _FILTEREDBLOCK
_DELIVERRESPONSE.oneofs_by_name['Type'].fields.append(
  _DELIVERRESPONSE.fields_by_name['status'])
_DELIVERRESPONSE.fields_by_name['status'].containing_oneof = _DELIVERRESPONSE.oneofs_by_name['Type']
_DELIVERRESPONSE.oneofs_by_name['Type'].fields.append(
  _DELIVERRESPONSE.fields_by_name['block'])
_DELIVERRESPONSE.fields_by_name['block'].containing_oneof = _DELIVERRESPONSE.oneofs_by_name['Type']
_DELIVERRESPONSE.oneofs_by_name['Type'].fields.append(
  _DELIVERRESPONSE.fields_by_name['filtered_block'])
_DELIVERRESPONSE.fields_by_name['filtered_block'].containing_oneof = _DELIVERRESPONSE.oneofs_by_name['Type']
DESCRIPTOR.message_types_by_name['FilteredBlock'] = _FILTEREDBLOCK
DESCRIPTOR.message_types_by_name['FilteredTransaction'] = _FILTEREDTRANSACTION
DESCRIPTOR.message_types_by_name['FilteredTransactionActions'] = _FILTEREDTRANSACTIONACTIONS
DESCRIPTOR.message_types_by_name['FilteredChaincodeAction'] = _FILTEREDCHAINCODEACTION
DESCRIPTOR.message_types_by_name['DeliverResponse'] = _DELIVERRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FilteredBlock = _reflection.GeneratedProtocolMessageType('FilteredBlock', (_message.Message,), dict(
  DESCRIPTOR = _FILTEREDBLOCK,
  __module__ = 'hfc.protos.peer.events_pb2'
  # @@protoc_insertion_point(class_scope:protos.FilteredBlock)
  ))
_sym_db.RegisterMessage(FilteredBlock)

FilteredTransaction = _reflection.GeneratedProtocolMessageType('FilteredTransaction', (_message.Message,), dict(
  DESCRIPTOR = _FILTEREDTRANSACTION,
  __module__ = 'hfc.protos.peer.events_pb2'
  # @@protoc_insertion_point(class_scope:protos.FilteredTransaction)
  ))
_sym_db.RegisterMessage(FilteredTransaction)

FilteredTransactionActions = _reflection.GeneratedProtocolMessageType('FilteredTransactionActions', (_message.Message,), dict(
  DESCRIPTOR = _FILTEREDTRANSACTIONACTIONS,
  __module__ = 'hfc.protos.peer.events_pb2'
  # @@protoc_insertion_point(class_scope:protos.FilteredTransactionActions)
  ))
_sym_db.RegisterMessage(FilteredTransactionActions)

FilteredChaincodeAction = _reflection.GeneratedProtocolMessageType('FilteredChaincodeAction', (_message.Message,), dict(
  DESCRIPTOR = _FILTEREDCHAINCODEACTION,
  __module__ = 'hfc.protos.peer.events_pb2'
  # @@protoc_insertion_point(class_scope:protos.FilteredChaincodeAction)
  ))
_sym_db.RegisterMessage(FilteredChaincodeAction)

DeliverResponse = _reflection.GeneratedProtocolMessageType('DeliverResponse', (_message.Message,), dict(
  DESCRIPTOR = _DELIVERRESPONSE,
  __module__ = 'hfc.protos.peer.events_pb2'
  # @@protoc_insertion_point(class_scope:protos.DeliverResponse)
  ))
_sym_db.RegisterMessage(DeliverResponse)


DESCRIPTOR._options = None

_DELIVER = _descriptor.ServiceDescriptor(
  name='Deliver',
  full_name='protos.Deliver',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=803,
  serialized_end=940,
  methods=[
  _descriptor.MethodDescriptor(
    name='Deliver',
    full_name='protos.Deliver.Deliver',
    index=0,
    containing_service=None,
    input_type=hfc_dot_protos_dot_common_dot_common__pb2._ENVELOPE,
    output_type=_DELIVERRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeliverFiltered',
    full_name='protos.Deliver.DeliverFiltered',
    index=1,
    containing_service=None,
    input_type=hfc_dot_protos_dot_common_dot_common__pb2._ENVELOPE,
    output_type=_DELIVERRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_DELIVER)

DESCRIPTOR.services_by_name['Deliver'] = _DELIVER

# @@protoc_insertion_point(module_scope)



# Copyright IBM Corp. 2016 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import time
import unittest

from hfc.fabric_ca.caservice import ca_service
from hfc.fabric.user import User
from hfc.util.keyvaluestore import file_key_value_store
from test.integration.utils import cli_call

USER_ID = 'user'
USER_PASSWD = 'userpw'


def get_submitter(store):
    ca = ca_service()
    user = User(USER_ID, USER_PASSWD, store)
    enrollment = ca.enroll(USER_ID, USER_PASSWD)
    user.enrollment = enrollment

    return user


class UserTest(unittest.TestCase):
    def setUp(self):
        self.gopath_bak = os.environ.get('GOPATH', '')
        gopath = os.path.normpath(os.path.join(os.path.dirname(__file__),
                                               "../fixtures/chaincode"))
        os.environ['GOPATH'] = os.path.abspath(gopath)
        self.base_path = '/tmp/fabric-sdk-py'
        self.kv_store_path = os.path.join(self.base_path, 'key-value-store')
        self.compose_file_path = os.path.normpath(
            os.path.join(os.path.dirname(__file__),
                         "../fixtures/chaincode/docker-compose-simple.yml")
        )
        self.start_test_env()

    def tearDown(self):
        if self.gopath_bak:
            os.environ['GOPATH'] = self.gopath_bak
        self.shutdown_test_env()

    def start_test_env(self):
        cli_call(["docker-compose", "-f", self.compose_file_path, "up", "-d"])

    def shutdown_test_env(self):
        cli_call(["docker-compose", "-f", self.compose_file_path, "down"])

    def test_get_submitter(self):
        time.sleep(5)

        submitter = get_submitter(file_key_value_store(self.kv_store_path))

        self.assertTrue(submitter.is_enrolled())

        # test the identity object carrying
        self.assertTrue(submitter.enrollment.cert.
                        startswith(b"-----BEGIN CERTIFICATE-----"))


if __name__ == '__main__':
    unittest.main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/token/transaction.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/token/transaction.proto',
  package='token',
  syntax='proto3',
  serialized_options=_b('Z*github.com/hyperledger/fabric/protos/token'),
  serialized_pb=_b('\n\"hfc/protos/token/transaction.proto\x12\x05token\"M\n\x10TokenTransaction\x12/\n\x0cplain_action\x18\x01 \x01(\x0b\x32\x17.token.PlainTokenActionH\x00\x42\x08\n\x06\x61\x63tion\"\x8b\x02\n\x10PlainTokenAction\x12*\n\x0cplain_import\x18\x01 \x01(\x0b\x32\x12.token.PlainImportH\x00\x12.\n\x0eplain_transfer\x18\x02 \x01(\x0b\x32\x14.token.PlainTransferH\x00\x12,\n\x0cplain_redeem\x18\x03 \x01(\x0b\x32\x14.token.PlainTransferH\x00\x12,\n\rplain_approve\x18\x04 \x01(\x0b\x32\x13.token.PlainApproveH\x00\x12\x37\n\x13plain_transfer_From\x18\x05 \x01(\x0b\x32\x18.token.PlainTransferFromH\x00\x42\x06\n\x04\x64\x61ta\"2\n\x0bPlainImport\x12#\n\x07outputs\x18\x01 \x03(\x0b\x32\x12.token.PlainOutput\"T\n\rPlainTransfer\x12\x1e\n\x06inputs\x18\x01 \x03(\x0b\x32\x0e.token.InputId\x12#\n\x07outputs\x18\x02 \x03(\x0b\x32\x12.token.PlainOutput\"\x8a\x01\n\x0cPlainApprove\x12\x1e\n\x06inputs\x18\x01 \x03(\x0b\x32\x0e.token.InputId\x12\x36\n\x11\x64\x65legated_outputs\x18\x02 \x03(\x0b\x32\x1b.token.PlainDelegatedOutput\x12\"\n\x06output\x18\x03 \x01(\x0b\x32\x12.token.PlainOutput\"\x8f\x01\n\x11PlainTransferFrom\x12\x1e\n\x06inputs\x18\x01 \x03(\x0b\x32\x0e.token.InputId\x12#\n\x07outputs\x18\x02 \x03(\x0b\x32\x12.token.PlainOutput\x12\x35\n\x10\x64\x65legated_output\x18\x03 \x01(\x0b\x32\x1b.token.PlainDelegatedOutput\"<\n\x0bPlainOutput\x12\r\n\x05owner\x18\x01 \x01(\x0c\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x10\n\x08quantity\x18\x03 \x01(\x04\"\'\n\x07InputId\x12\r\n\x05tx_id\x18\x01 \x01(\t\x12\r\n\x05index\x18\x02 \x01(\r\"Y\n\x14PlainDelegatedOutput\x12\r\n\x05owner\x18\x01 \x01(\x0c\x12\x12\n\ndelegatees\x18\x02 \x03(\x0c\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x10\n\x08quantity\x18\x04 \x01(\x04\x42,Z*github.com/hyperledger/fabric/protos/tokenb\x06proto3')
)




_TOKENTRANSACTION = _descriptor.Descriptor(
  name='TokenTransaction',
  full_name='token.TokenTransaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='plain_action', full_name='token.TokenTransaction.plain_action', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='action', full_name='token.TokenTransaction.action',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=45,
  serialized_end=122,
)


_PLAINTOKENACTION = _descriptor.Descriptor(
  name='PlainTokenAction',
  full_name='token.PlainTokenAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='plain_import', full_name='token.PlainTokenAction.plain_import', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='plain_transfer', full_name='token.PlainTokenAction.plain_transfer', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='plain_redeem', full_name='token.PlainTokenAction.plain_redeem', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='plain_approve', full_name='token.PlainTokenAction.plain_approve', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='plain_transfer_From', full_name='token.PlainTokenAction.plain_transfer_From', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='token.PlainTokenAction.data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=125,
  serialized_end=392,
)


_PLAINIMPORT = _descriptor.Descriptor(
  name='PlainImport',
  full_name='token.PlainImport',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='outputs', full_name='token.PlainImport.outputs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=394,
  serialized_end=444,
)


_PLAINTRANSFER = _descriptor.Descriptor(
  name='PlainTransfer',
  full_name='token.PlainTransfer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='inputs', full_name='token.PlainTransfer.inputs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='outputs', full_name='token.PlainTransfer.outputs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=446,
  serialized_end=530,
)


_PLAINAPPROVE = _descriptor.Descriptor(
  name='PlainApprove',
  full_name='token.PlainApprove',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='inputs', full_name='token.PlainApprove.inputs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='delegated_outputs', full_name='token.PlainApprove.delegated_outputs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='output', full_name='token.PlainApprove.output', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=533,
  serialized_end=671,
)


_PLAINTRANSFERFROM = _descriptor.Descriptor(
  name='PlainTransferFrom',
  full_name='token.PlainTransferFrom',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='inputs', full_name='token.PlainTransferFrom.inputs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='outputs', full_name='token.PlainTransferFrom.outputs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='delegated_output', full_name='token.PlainTransferFrom.delegated_output', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=674,
  serialized_end=817,
)


_PLAINOUTPUT = _descriptor.Descriptor(
  name='PlainOutput',
  full_name='token.PlainOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='owner', full_name='token.PlainOutput.owner', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='token.PlainOutput.type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='quantity', full_name='token.PlainOutput.quantity', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=819,
  serialized_end=879,
)


_INPUTID = _descriptor.Descriptor(
  name='InputId',
  full_name='token.InputId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tx_id', full_name='token.InputId.tx_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='index', full_name='token.InputId.index', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=881,
  serialized_end=920,
)


_PLAINDELEGATEDOUTPUT = _descriptor.Descriptor(
  name='PlainDelegatedOutput',
  full_name='token.PlainDelegatedOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='owner', full_name='token.PlainDelegatedOutput.owner', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='delegatees', full_name='token.PlainDelegatedOutput.delegatees', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='token.PlainDelegatedOutput.type', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='quantity', full_name='token.PlainDelegatedOutput.quantity', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=922,
  serialized_end=1011,
)

_TOKENTRANSACTION.fields_by_name['plain_action'].message_type = _PLAINTOKENACTION
_TOKENTRANSACTION.oneofs_by_name['action'].fields.append(
  _TOKENTRANSACTION.fields_by_name['plain_action'])
_TOKENTRANSACTION.fields_by_name['plain_action'].containing_oneof = _TOKENTRANSACTION.oneofs_by_name['action']
_PLAINTOKENACTION.fields_by_name['plain_import'].message_type = _PLAINIMPORT
_PLAINTOKENACTION.fields_by_name['plain_transfer'].message_type = _PLAINTRANSFER
_PLAINTOKENACTION.fields_by_name['plain_redeem'].message_type = _PLAINTRANSFER
_PLAINTOKENACTION.fields_by_name['plain_approve'].message_type = _PLAINAPPROVE
_PLAINTOKENACTION.fields_by_name['plain_transfer_From'].message_type = _PLAINTRANSFERFROM
_PLAINTOKENACTION.oneofs_by_name['data'].fields.append(
  _PLAINTOKENACTION.fields_by_name['plain_import'])
_PLAINTOKENACTION.fields_by_name['plain_import'].containing_oneof = _PLAINTOKENACTION.oneofs_by_name['data']
_PLAINTOKENACTION.oneofs_by_name['data'].fields.append(
  _PLAINTOKENACTION.fields_by_name['plain_transfer'])
_PLAINTOKENACTION.fields_by_name['plain_transfer'].containing_oneof = _PLAINTOKENACTION.oneofs_by_name['data']
_PLAINTOKENACTION.oneofs_by_name['data'].fields.append(
  _PLAINTOKENACTION.fields_by_name['plain_redeem'])
_PLAINTOKENACTION.fields_by_name['plain_redeem'].containing_oneof = _PLAINTOKENACTION.oneofs_by_name['data']
_PLAINTOKENACTION.oneofs_by_name['data'].fields.append(
  _PLAINTOKENACTION.fields_by_name['plain_approve'])
_PLAINTOKENACTION.fields_by_name['plain_approve'].containing_oneof = _PLAINTOKENACTION.oneofs_by_name['data']
_PLAINTOKENACTION.oneofs_by_name['data'].fields.append(
  _PLAINTOKENACTION.fields_by_name['plain_transfer_From'])
_PLAINTOKENACTION.fields_by_name['plain_transfer_From'].containing_oneof = _PLAINTOKENACTION.oneofs_by_name['data']
_PLAINIMPORT.fields_by_name['outputs'].message_type = _PLAINOUTPUT
_PLAINTRANSFER.fields_by_name['inputs'].message_type = _INPUTID
_PLAINTRANSFER.fields_by_name['outputs'].message_type = _PLAINOUTPUT
_PLAINAPPROVE.fields_by_name['inputs'].message_type = _INPUTID
_PLAINAPPROVE.fields_by_name['delegated_outputs'].message_type = _PLAINDELEGATEDOUTPUT
_PLAINAPPROVE.fields_by_name['output'].message_type = _PLAINOUTPUT
_PLAINTRANSFERFROM.fields_by_name['inputs'].message_type = _INPUTID
_PLAINTRANSFERFROM.fields_by_name['outputs'].message_type = _PLAINOUTPUT
_PLAINTRANSFERFROM.fields_by_name['delegated_output'].message_type = _PLAINDELEGATEDOUTPUT
DESCRIPTOR.message_types_by_name['TokenTransaction'] = _TOKENTRANSACTION
DESCRIPTOR.message_types_by_name['PlainTokenAction'] = _PLAINTOKENACTION
DESCRIPTOR.message_types_by_name['PlainImport'] = _PLAINIMPORT
DESCRIPTOR.message_types_by_name['PlainTransfer'] = _PLAINTRANSFER
DESCRIPTOR.message_types_by_name['PlainApprove'] = _PLAINAPPROVE
DESCRIPTOR.message_types_by_name['PlainTransferFrom'] = _PLAINTRANSFERFROM
DESCRIPTOR.message_types_by_name['PlainOutput'] = _PLAINOUTPUT
DESCRIPTOR.message_types_by_name['InputId'] = _INPUTID
DESCRIPTOR.message_types_by_name['PlainDelegatedOutput'] = _PLAINDELEGATEDOUTPUT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TokenTransaction = _reflection.GeneratedProtocolMessageType('TokenTransaction', (_message.Message,), dict(
  DESCRIPTOR = _TOKENTRANSACTION,
  __module__ = 'hfc.protos.token.transaction_pb2'
  # @@protoc_insertion_point(class_scope:token.TokenTransaction)
  ))
_sym_db.RegisterMessage(TokenTransaction)

PlainTokenAction = _reflection.GeneratedProtocolMessageType('PlainTokenAction', (_message.Message,), dict(
  DESCRIPTOR = _PLAINTOKENACTION,
  __module__ = 'hfc.protos.token.transaction_pb2'
  # @@protoc_insertion_point(class_scope:token.PlainTokenAction)
  ))
_sym_db.RegisterMessage(PlainTokenAction)

PlainImport = _reflection.GeneratedProtocolMessageType('PlainImport', (_message.Message,), dict(
  DESCRIPTOR = _PLAINIMPORT,
  __module__ = 'hfc.protos.token.transaction_pb2'
  # @@protoc_insertion_point(class_scope:token.PlainImport)
  ))
_sym_db.RegisterMessage(PlainImport)

PlainTransfer = _reflection.GeneratedProtocolMessageType('PlainTransfer', (_message.Message,), dict(
  DESCRIPTOR = _PLAINTRANSFER,
  __module__ = 'hfc.protos.token.transaction_pb2'
  # @@protoc_insertion_point(class_scope:token.PlainTransfer)
  ))
_sym_db.RegisterMessage(PlainTransfer)

PlainApprove = _reflection.GeneratedProtocolMessageType('PlainApprove', (_message.Message,), dict(
  DESCRIPTOR = _PLAINAPPROVE,
  __module__ = 'hfc.protos.token.transaction_pb2'
  # @@protoc_insertion_point(class_scope:token.PlainApprove)
  ))
_sym_db.RegisterMessage(PlainApprove)

PlainTransferFrom = _reflection.GeneratedProtocolMessageType('PlainTransferFrom', (_message.Message,), dict(
  DESCRIPTOR = _PLAINTRANSFERFROM,
  __module__ = 'hfc.protos.token.transaction_pb2'
  # @@protoc_insertion_point(class_scope:token.PlainTransferFrom)
  ))
_sym_db.RegisterMessage(PlainTransferFrom)

PlainOutput = _reflection.GeneratedProtocolMessageType('PlainOutput', (_message.Message,), dict(
  DESCRIPTOR = _PLAINOUTPUT,
  __module__ = 'hfc.protos.token.transaction_pb2'
  # @@protoc_insertion_point(class_scope:token.PlainOutput)
  ))
_sym_db.RegisterMessage(PlainOutput)

InputId = _reflection.GeneratedProtocolMessageType('InputId', (_message.Message,), dict(
  DESCRIPTOR = _INPUTID,
  __module__ = 'hfc.protos.token.transaction_pb2'
  # @@protoc_insertion_point(class_scope:token.InputId)
  ))
_sym_db.RegisterMessage(InputId)

PlainDelegatedOutput = _reflection.GeneratedProtocolMessageType('PlainDelegatedOutput', (_message.Message,), dict(
  DESCRIPTOR = _PLAINDELEGATEDOUTPUT,
  __module__ = 'hfc.protos.token.transaction_pb2'
  # @@protoc_insertion_point(class_scope:token.PlainDelegatedOutput)
  ))
_sym_db.RegisterMessage(PlainDelegatedOutput)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import copy

from lark import Lark
from lark import Transformer

s2d_grammar = r"""
    ?value: e
          | role
          | DIGIT -> number

    dot: "."
    dash: "-"
    name: /[\w\d\-\$\&\+\,\:\;\=\?\@\#\|\<\>\^\*\(\)\%\!]+/
    mspid: WORD
    role: "'" name dot mspid "'"
    or: "OR"
    and: "AND"
    outof: "OutOf"
    logic: or | and | outof
    e : logic "(" [value ("," value)*] ")"

    %import common.WORD
    %import common.LETTER
    %import common.DIGIT
    %import common.WS
    %ignore WS

    """


class String2Dict(Transformer):

    def __init__(self):
        self.id = 0
        self.roles = []

    def unique_list_of_dict(self, list_of_dict):
        unique_l = []

        for item in list_of_dict:
            if item not in unique_l:
                unique_l.append(item)

        return unique_l

    def get_logic(self, args, n):

        identities = []
        policies = []

        for i, v in enumerate(args):
            if 'policy' in v:
                identities += v['identities']
                policies.append(v['policy'])
            else:
                identities.append({"role": {
                    'name': v['name'],
                    'mspId': v['mspId']
                }})
                policies.append({"signed-by": v['id']})

        return {
            "identities": self.unique_list_of_dict(identities),
            "policy": {
                f"{n}-of": policies
            }
        }

    def get_outof(self, items):
        digit, *args = items
        return self.get_logic(args, digit)

    def name(self, items):
        return ''.join(items)

    def role(self, items):
        mspId, dot, name = items

        # check if identity already exists in self.identities
        for role in self.roles:
            if role['name'] == name \
                    and role['mspId'] == mspId:
                break
        else:
            role = {"name": name,
                    "mspId": mspId,
                    "id": self.id}
            self.id += 1
            self.roles.append(role)

        return role

    def logic(self, items):
        logic, = items
        return logic.data

    def dot(self, *args):
        return '.'

    def dash(self, *args):
        return '-'

    def mspid(self, items):
        return str(items[0])

    def number(self, items):
        return int(items[0])

    def e(self, items):
        logic, *args = items

        if logic == 'or':
            return self.get_logic(args, 1)
        elif logic == 'and':
            return self.get_logic(args, len(args))
        elif logic == 'outof':
            return self.get_outof(args)

        return items


class Dict2String(object):
    roles = []

    def get_policy(self, policy):
        policy_key = list(policy.keys())[0]
        n = policy_key.split('-of')[0]

        roles = []
        subpolicies = []

        if isinstance(policy[policy_key], list):
            for p in policy[policy_key]:
                key = list(p.keys())[0]
                if key == 'signed-by':
                    r = self.roles[p[key]]
                    roles.append(r)
                else:
                    p = self.get_policy(p)
                    subpolicies.append(p)
        else:
            n = 1
            subpolicies = [self.roles[policy[policy_key]]]

        return f"OutOf({n}, {', '.join(roles)}{', '.join(subpolicies)})"

    def parse(self, policy):
        p = copy.deepcopy(policy)

        self.roles = [f"'{x['role']['mspId']}.{x['role']['name']}'"
                      for x in p['identities']]

        return self.get_policy(p['policy'])


def s2d():
    # new instance for resetting local variables on each call
    transformer = String2Dict()
    return Lark(s2d_grammar, start='value', parser='lalr',
                transformer=transformer)


d2s = Dict2String()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/transientstore/transientstore.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hfc.protos.ledger.rwset import rwset_pb2 as hfc_dot_protos_dot_ledger_dot_rwset_dot_rwset__pb2
from hfc.protos.common import collection_pb2 as hfc_dot_protos_dot_common_dot_collection__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/transientstore/transientstore.proto',
  package='transientstore',
  syntax='proto3',
  serialized_options=_b('\n,org.hyperledger.fabric.protos.transientstoreZ3github.com/hyperledger/fabric/protos/transientstore'),
  serialized_pb=_b('\n.hfc/protos/transientstore/transientstore.proto\x12\x0etransientstore\x1a#hfc/protos/ledger/rwset/rwset.proto\x1a\"hfc/protos/common/collection.proto\"\xa2\x02\n\x1fTxPvtReadWriteSetWithConfigInfo\x12\x13\n\x0b\x65ndorsed_at\x18\x01 \x01(\x04\x12+\n\tpvt_rwset\x18\x02 \x01(\x0b\x32\x18.rwset.TxPvtReadWriteSet\x12\x62\n\x12\x63ollection_configs\x18\x03 \x03(\x0b\x32\x46.transientstore.TxPvtReadWriteSetWithConfigInfo.CollectionConfigsEntry\x1aY\n\x16\x43ollectionConfigsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12.\n\x05value\x18\x02 \x01(\x0b\x32\x1f.common.CollectionConfigPackage:\x02\x38\x01\x42\x63\n,org.hyperledger.fabric.protos.transientstoreZ3github.com/hyperledger/fabric/protos/transientstoreb\x06proto3')
  ,
  dependencies=[hfc_dot_protos_dot_ledger_dot_rwset_dot_rwset__pb2.DESCRIPTOR,hfc_dot_protos_dot_common_dot_collection__pb2.DESCRIPTOR,])




_TXPVTREADWRITESETWITHCONFIGINFO_COLLECTIONCONFIGSENTRY = _descriptor.Descriptor(
  name='CollectionConfigsEntry',
  full_name='transientstore.TxPvtReadWriteSetWithConfigInfo.CollectionConfigsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='transientstore.TxPvtReadWriteSetWithConfigInfo.CollectionConfigsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='transientstore.TxPvtReadWriteSetWithConfigInfo.CollectionConfigsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=341,
  serialized_end=430,
)

_TXPVTREADWRITESETWITHCONFIGINFO = _descriptor.Descriptor(
  name='TxPvtReadWriteSetWithConfigInfo',
  full_name='transientstore.TxPvtReadWriteSetWithConfigInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='endorsed_at', full_name='transientstore.TxPvtReadWriteSetWithConfigInfo.endorsed_at', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pvt_rwset', full_name='transientstore.TxPvtReadWriteSetWithConfigInfo.pvt_rwset', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='collection_configs', full_name='transientstore.TxPvtReadWriteSetWithConfigInfo.collection_configs', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TXPVTREADWRITESETWITHCONFIGINFO_COLLECTIONCONFIGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=140,
  serialized_end=430,
)

_TXPVTREADWRITESETWITHCONFIGINFO_COLLECTIONCONFIGSENTRY.fields_by_name['value'].message_type = hfc_dot_protos_dot_common_dot_collection__pb2._COLLECTIONCONFIGPACKAGE
_TXPVTREADWRITESETWITHCONFIGINFO_COLLECTIONCONFIGSENTRY.containing_type = _TXPVTREADWRITESETWITHCONFIGINFO
_TXPVTREADWRITESETWITHCONFIGINFO.fields_by_name['pvt_rwset'].message_type = hfc_dot_protos_dot_ledger_dot_rwset_dot_rwset__pb2._TXPVTREADWRITESET
_TXPVTREADWRITESETWITHCONFIGINFO.fields_by_name['collection_configs'].message_type = _TXPVTREADWRITESETWITHCONFIGINFO_COLLECTIONCONFIGSENTRY
DESCRIPTOR.message_types_by_name['TxPvtReadWriteSetWithConfigInfo'] = _TXPVTREADWRITESETWITHCONFIGINFO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TxPvtReadWriteSetWithConfigInfo = _reflection.GeneratedProtocolMessageType('TxPvtReadWriteSetWithConfigInfo', (_message.Message,), dict(

  CollectionConfigsEntry = _reflection.GeneratedProtocolMessageType('CollectionConfigsEntry', (_message.Message,), dict(
    DESCRIPTOR = _TXPVTREADWRITESETWITHCONFIGINFO_COLLECTIONCONFIGSENTRY,
    __module__ = 'hfc.protos.transientstore.transientstore_pb2'
    # @@protoc_insertion_point(class_scope:transientstore.TxPvtReadWriteSetWithConfigInfo.CollectionConfigsEntry)
    ))
  ,
  DESCRIPTOR = _TXPVTREADWRITESETWITHCONFIGINFO,
  __module__ = 'hfc.protos.transientstore.transientstore_pb2'
  # @@protoc_insertion_point(class_scope:transientstore.TxPvtReadWriteSetWithConfigInfo)
  ))
_sym_db.RegisterMessage(TxPvtReadWriteSetWithConfigInfo)
_sym_db.RegisterMessage(TxPvtReadWriteSetWithConfigInfo.CollectionConfigsEntry)


DESCRIPTOR._options = None
_TXPVTREADWRITESETWITHCONFIGINFO_COLLECTIONCONFIGSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/peer/chaincode_event.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/peer/chaincode_event.proto',
  package='protos',
  syntax='proto3',
  serialized_options=_b('\n\"org.hyperledger.fabric.protos.peerB\025ChaincodeEventPackageZ)github.com/hyperledger/fabric/protos/peer'),
  serialized_pb=_b('\n%hfc/protos/peer/chaincode_event.proto\x12\x06protos\"Z\n\x0e\x43haincodeEvent\x12\x14\n\x0c\x63haincode_id\x18\x01 \x01(\t\x12\r\n\x05tx_id\x18\x02 \x01(\t\x12\x12\n\nevent_name\x18\x03 \x01(\t\x12\x0f\n\x07payload\x18\x04 \x01(\x0c\x42\x66\n\"org.hyperledger.fabric.protos.peerB\x15\x43haincodeEventPackageZ)github.com/hyperledger/fabric/protos/peerb\x06proto3')
)




_CHAINCODEEVENT = _descriptor.Descriptor(
  name='ChaincodeEvent',
  full_name='protos.ChaincodeEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chaincode_id', full_name='protos.ChaincodeEvent.chaincode_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tx_id', full_name='protos.ChaincodeEvent.tx_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='event_name', full_name='protos.ChaincodeEvent.event_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='protos.ChaincodeEvent.payload', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=49,
  serialized_end=139,
)

DESCRIPTOR.message_types_by_name['ChaincodeEvent'] = _CHAINCODEEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ChaincodeEvent = _reflection.GeneratedProtocolMessageType('ChaincodeEvent', (_message.Message,), dict(
  DESCRIPTOR = _CHAINCODEEVENT,
  __module__ = 'hfc.protos.peer.chaincode_event_pb2'
  # @@protoc_insertion_point(class_scope:protos.ChaincodeEvent)
  ))
_sym_db.RegisterMessage(ChaincodeEvent)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# SPDX-License-Identifier: Apache-2.0

import asyncio
import time
from asyncio import shield

import docker
import logging
import unittest

from hfc.fabric.channel.channel import SYSTEM_CHANNEL_NAME
from hfc.util.policies import s2d

from test.integration.utils import BaseTestCase

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

CC_PATH = 'github.com/example_cc_with_event'
CC_NAME = 'example_cc_with_event'
CC_VERSION = '1.0'


class E2eTest(BaseTestCase):

    def setUp(self):
        super(E2eTest, self).setUp()

    def tearDown(self):
        super(E2eTest, self).tearDown()

    async def channel_create(self):
        """
        Create an channel for further testing.

        :return:
        """
        logger.info(f"E2E: Channel creation start: name={self.channel_name}")

        # By default, self.user is the admin of org1
        response = await self.client.channel_create(
            'orderer.example.com',
            self.channel_name,
            self.user,
            config_yaml=self.config_yaml,
            channel_profile=self.channel_profile)
        self.assertTrue(response)

        logger.info(f"E2E: Channel creation done: name={self.channel_name}")

    async def channel_join(self):
        """
        Join peers of two orgs into an existing channels

        :return:
        """

        logger.info(f"E2E: Channel join start: name={self.channel_name}")

        # channel must already exist when to join
        channel = self.client.get_channel(self.channel_name)
        self.assertIsNotNone(channel)

        orgs = ["org1.example.com", "org2.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, 'Admin')
            response = await self.client.channel_join(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
                orderer='orderer.example.com'
            )
            self.assertTrue(response)
            # Verify the ledger exists now in the peer node
            dc = docker.from_env()
            for peer in ['peer0', 'peer1']:
                peer0_container = dc.containers.get(peer + '.' + org)
                code, output = peer0_container.exec_run(
                    'test -f '
                    '/var/hyperledger/production/ledgersData/chains/'
                    f'chains/{self.channel_name}'
                    '/blockfile_000000')
                self.assertEqual(code, 0, "Local ledger not exists")

        logger.info(f"E2E: Channel join done: name={self.channel_name}")

    async def chaincode_install(self):
        """
        Test installing an example chaincode to peer
        """
        logger.info("E2E: Chaincode install start")
        cc = f'/var/hyperledger/production/chaincodes/{CC_NAME}.{CC_VERSION}'

        # uncomment for testing with packaged_cc

        # create packaged chaincode before for having same id
        # code_package = package_chaincode(CC_PATH, CC_TYPE_GOLANG)

        orgs = ["org1.example.com", "org2.example.com"]
        for org in orgs:

            # simulate possible different chaincode archive based on timestamp
            time.sleep(2)

            org_admin = self.client.get_user(org, "Admin")
            responses = await self.client.chaincode_install(
                requestor=org_admin,
                peers=['peer0.' + org, 'peer1.' + org],
                cc_path=CC_PATH,
                cc_name=CC_NAME,
                cc_version=CC_VERSION,
                # packaged_cc=code_package
            )
            self.assertTrue(responses)
            # Verify the cc pack exists now in the peer node
            dc = docker.from_env()
            for peer in ['peer0', 'peer1']:
                peer_container = dc.containers.get(peer + '.' + org)
                code, output = peer_container.exec_run(f'test -f {cc}')
                self.assertEqual(code, 0, "chaincodes pack not exists")

        logger.info("E2E: chaincode install done")

    def chaincode_install_fail(self):
        pass

    async def chaincode_instantiate(self):
        """
        Test instantiating an example chaincode to peer
        """
        logger.info("E2E: Chaincode instantiation start")

        org = "org1.example.com"
        args = ['a', '200', 'b', '300']

        # policy = s2d().parse("OR('Org1MSP.member', 'Org1MSP.admin')")
        policy = s2d().parse("OR('Org1MSP.member')")

        org_admin = self.client.get_user(org, "Admin")
        response = await self.client.chaincode_instantiate(
            requestor=org_admin,
            channel_name=self.channel_name,
            peers=['peer0.' + org],
            args=args,
            cc_name=CC_NAME,
            cc_version=CC_VERSION,
            cc_endorsement_policy=policy,
            wait_for_event=True
        )
        logger.info(
            "E2E: Chaincode instantiation response {}".format(response))
        policy = {
            'version': 0,
            'rule': {'n_out_of': {
                'n': 1,
                'rules': [
                    {'signed_by': 0},
                    # {'signed_by': 1}
                ]}
            },
            'identities': [
                {
                    'principal_classification': 'ROLE',
                    'principal': {
                        'msp_identifier': 'Org1MSP',
                        'role': 'MEMBER'
                    }
                },
                # {
                #     'principal_classification': 'ROLE',
                #     'principal': {
                #         'msp_identifier': 'Org1MSP',
                #         'role': 'ADMIN'
                #     }
                # },
            ]
        }
        self.assertEqual(response['name'], CC_NAME)
        self.assertEqual(response['version'], CC_VERSION)
        self.assertEqual(response['policy'], policy)
        logger.info("E2E: chaincode instantiation done")

    async def chaincode_invoke(self):
        """
        Test invoking an example chaincode to peer

        :return:
        """
        logger.info("E2E: Chaincode invoke start")

        orgs = ["org1.example.com"]
        args = ['a', 'b', '100']
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")

            response = await self.client.chaincode_invoke(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer1.' + org],
                args=args,
                cc_name=CC_NAME,
                wait_for_event=True,
                wait_for_event_timeout=120,
                cc_pattern="^invoked*"  # for chaincode event
            )
            self.assertEqual(response, '400')  # 300 + 100

        logger.info("E2E: chaincode invoke done")

    async def chaincode_invoke_fail(self):
        """
        Test invoking an example chaincode to peer

        :return:
        """
        logger.info("E2E: Chaincode invoke fail start")

        orgs = ["org2.example.com"]
        args = ['a', 'b', '100']
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            with self.assertRaises(Exception) as e:
                await self.client.chaincode_invoke(
                    requestor=org_admin,
                    channel_name=self.channel_name,
                    peers=['peer1.' + org],
                    args=args,
                    cc_name=CC_NAME,
                    wait_for_event=True,
                    wait_for_event_timeout=120,
                    cc_pattern="^invoked*"  # for chaincode event
                )
            self.assertEqual(e.exception.args[0],
                             ['ENDORSEMENT_POLICY_FAILURE'])

        logger.info("E2E: chaincode invoke fail done")

    async def chaincode_channel_event_hub(self):
        """
        Test invoking an example chaincode to peer

        :return:
        """
        logger.info("E2E: Chaincode Channel Event Hub test start")

        def onEvent(cc_event, block_number, tx_id, tx_status):
            self.ceh.unregisterChaincodeEvent(self.cr1)
            self.ceh.unregisterChaincodeEvent(self.cr2)
            self.ceh.unregisterChaincodeEvent(self.cr3)
            self.ceh.disconnect()

        orgs = ["org1.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            # register extra chaincode event
            channel = self.client.get_channel(self.channel_name)
            target_peer = self.client.get_peer('peer1.' + org)
            self.ceh = channel.newChannelEventHub(target_peer, org_admin)
            stream = self.ceh.connect()
            self.cr1 = self.ceh.registerChaincodeEvent(CC_NAME, 'invoked')
            self.cr2 = self.ceh.registerChaincodeEvent(CC_NAME, 'invoked')
            self.cr3 = self.ceh.registerChaincodeEvent(CC_NAME, 'invoked',
                                                       onEvent=onEvent)

            await asyncio.wait_for(asyncio.gather(stream,
                                                  return_exceptions=True),
                                   timeout=120)

        logger.info("E2E: Chaincode Channel Event Hub test done")

    async def chaincode_query(self, orgs=None):
        """
        Test invoking an example chaincode to peer

        :return:
        """
        logger.info("E2E: Chaincode query start")

        if orgs is None:
            orgs = ["org1.example.com"]

        args = ['b']
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            response = await self.client.chaincode_query(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org],
                args=args,
                cc_name=CC_NAME
            )
            self.assertEqual(response, '400')  # 300 + 100

        logger.info("E2E: chaincode query done")

    async def query_installed_chaincodes(self):
        """
        Test query installed chaincodes on peer

        :return:
        """
        logger.info("E2E: Query installed chaincode start")

        orgs = ["org1.example.com", "org2.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            responses = await self.client.query_installed_chaincodes(
                requestor=org_admin,
                peers=['peer0.' + org, 'peer1.' + org],
            )
            self.assertEqual(
                responses[0].chaincodes[0].name, CC_NAME, "Query failed")
            self.assertEqual(
                responses[0].chaincodes[0].version, CC_VERSION, "Query failed")
            self.assertEqual(
                responses[0].chaincodes[0].path, CC_PATH, "Query failed")

        logger.info("E2E: Query installed chaincode done")

    async def query_channels(self):
        """
        Test querying channel

        :return:
        """
        logger.info("E2E: Query channel start")

        orgs = ["org1.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            response = await self.client.query_channels(
                requestor=org_admin,
                peers=['peer0.' + org, 'peer1.' + org],
            )
            self.assertEqual(
                response.channels[0].channel_id,
                self.channel_name,
                "Query failed")

        logger.info("E2E: Query channel done")

    async def query_info(self):
        """
        Test querying information on the state of the Channel

        :return:
        """
        logger.info("E2E: Query info start")

        orgs = ["org1.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            response = await self.client.query_info(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
            )
            self.assertEqual(
                response.height,
                4,
                "Query failed")

        logger.info("E2E: Query info done")

    async def query_block_by_txid(self):
        """
        Test querying block by tx id

        :return:
        """
        logger.info("E2E: Query block by tx id start")

        orgs = ["org1.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")

            response = await self.client.query_info(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
            )

            response = await self.client.query_block_by_hash(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
                block_hash=response.currentBlockHash
            )

            tx_id = response.get('data').get('data')[0].get(
                'payload').get('header').get(
                'channel_header').get('tx_id')

            response = await self.client.query_block_by_txid(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
                tx_id=tx_id
            )

            self.assertEqual(
                response.get('data').get('data')[0].get(
                    'payload').get('header').get(
                    'channel_header').get('tx_id'),
                tx_id,
                "Query failed")

        logger.info("E2E: Query block by tx id done")

    async def query_block_by_hash(self):
        """
        Test querying block by block hash

        :return:
        """
        logger.info("E2E: Query block by block hash start")

        orgs = ["org1.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")

            response = await self.client.query_info(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
            )

            previous_block_hash = response.previousBlockHash
            current_block_hash = response.currentBlockHash
            response = await self.client.query_block_by_hash(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
                block_hash=current_block_hash
            )

            self.assertEqual(
                response['header']['previous_hash'].decode('utf-8'),
                previous_block_hash.hex(),
                "Query failed")

        logger.info("E2E: Query block by block hash done")

    async def query_block(self):
        """
        Test querying block by block number

        :return:
        """
        logger.info("E2E: Query block by block number start")

        orgs = ["org1.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            response = await self.client.query_block(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
                block_number='0'
            )
            self.assertEqual(
                response['header']['number'],
                0,
                "Query failed")
            self.blockheader = response['header']

        logger.info("E2E: Query block by block number done")

    async def query_transaction(self):
        """
        Test querying transaction by tx id

        :return:
        """
        logger.info("E2E: Query transaction by tx id start")
        orgs = ["org1.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")

            response = await self.client.query_info(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
            )

            response = await self.client.query_block_by_hash(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
                block_hash=response.currentBlockHash
            )

            tx_id = response.get('data').get('data')[0].get(
                'payload').get('header').get(
                'channel_header').get('tx_id')

            response = await self.client.query_transaction(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
                tx_id=tx_id
            )

            self.assertEqual(
                response.get('transaction_envelope').get('payload').get(
                    'header').get('channel_header').get('channel_id'),
                self.channel_name,
                "Query failed")

        logger.info("E2E: Query transaction by tx id done")

    async def query_instantiated_chaincodes(self):
        """
        Test query instantiated chaincodes on peer

        :return:
        """
        logger.info("E2E: Query instantiated chaincode start")

        orgs = ["org1.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")

            responses = await self.client.query_instantiated_chaincodes(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
            )
            self.assertTrue(len(responses) >= 1)
            self.assertEqual(
                responses[0].chaincodes[0].name, CC_NAME, "Query failed")
            self.assertEqual(
                responses[0].chaincodes[0].version, CC_VERSION, "Query failed")
            self.assertEqual(
                responses[0].chaincodes[0].path, CC_PATH, "Query failed")

        logger.info("E2E: Query installed chaincode done")

    async def get_channel_config(self):
        """
        Test get channel config on peer

        :return:
        """
        logger.info(f"E2E: Get channel {self.channel_name} config start")

        orgs = ["org1.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            responses = await self.client.get_channel_config(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org]
            )
            self.assertEqual(responses[0].config.sequence,
                             1, "Get Config Failed")

        logger.info("E2E: Query installed chaincode done")

    async def get_channel_config_with_orderer(self,
                                              chname=SYSTEM_CHANNEL_NAME):
        """
        Test get channel config on orderer
         :return:
        """
        logger.info(f"E2E: Get channel {chname} config start")

        orgs = ["orderer.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            response = await self.client.get_channel_config_with_orderer(
                orderer='orderer.example.com',
                requestor=org_admin,
                channel_name=chname,
            )
            self.assertEqual(response['config']['sequence'],
                             '0', "Get Config Failed")

        logger.info(f"E2E: Get channel {chname} config done")

    def onFilteredEvent(self, block):
        self.filtered_blocks.append(block)

    async def get_filtered_block_events(self):

        org = 'org1.example.com'
        peer = self.client.get_peer('peer0.' + org)

        org_admin = self.client.get_user(org, 'Admin')
        channel = self.client.get_channel(self.channel_name)
        channel_event_hub = channel.newChannelEventHub(peer, org_admin)
        stream = channel_event_hub.connect(filtered=True,
                                           start='oldest', stop='newest')

        self.filtered_blocks = []
        channel_event_hub.registerBlockEvent(unregister=False,
                                             onEvent=self.onFilteredEvent)

        try:
            await shield(stream)
        except Exception:
            pass

        channel_event_hub.disconnect()

        self.assertEqual(len(self.filtered_blocks), 4)

        block = self.filtered_blocks[0]
        self.assertEqual(block['number'], 0)
        self.assertEqual(block['channel_id'], self.channel_name)

        filtered_transaction = block['filtered_transactions'][0]
        self.assertEqual(filtered_transaction['tx_validation_code'], 'VALID')
        self.assertEqual(filtered_transaction['txid'], '')
        self.assertEqual(filtered_transaction['type'], 'CONFIG')

    def onFullEvent(self, block):
        self.blocks.append(block)

    async def get_full_block_events(self):

        org = 'org1.example.com'
        peer = self.client.get_peer('peer0.' + org)

        org_admin = self.client.get_user(org, 'Admin')
        channel = self.client.get_channel(self.channel_name)
        channel_event_hub = channel.newChannelEventHub(peer, org_admin)
        stream = channel_event_hub.connect(start='oldest',
                                           stop='newest', filtered=False)

        self.blocks = []
        channel_event_hub.registerBlockEvent(unregister=False,
                                             onEvent=self.onFullEvent)

        try:
            await shield(stream)
        except Exception:
            pass

        channel_event_hub.disconnect()

        self.assertEqual(len(self.blocks), 4)

        block = self.blocks[0]
        self.assertEqual(block['header']['number'], 0)

        block = self.blocks[2]
        self.assertEqual(block['header']['number'], 2)
        action = block['data']['data'][0]['payload']['data']['actions'][0]
        ppl_r_p = action['payload']['action']['proposal_response_payload']
        events_obj = ppl_r_p['extension']['events']
        self.assertEqual(events_obj['event_name'], 'invoked')
        self.assertEqual(events_obj['chaincode_id'], CC_NAME)
        self.assertEqual(events_obj['payload'], b'400')

    def onTxEvent(self, tx_id, status, block_number):

        o = {
            'status': status,
            'block_number': block_number
        }

        if tx_id == 'all':
            if tx_id not in self.txs:
                self.txs[tx_id] = []
            self.txs[tx_id] += [o]
        else:
            self.txs[tx_id] = o

    async def get_tx_events(self):

        org = 'org1.example.com'
        peer = self.client.get_peer('peer0.' + org)

        org_admin = self.client.get_user(org, 'Admin')
        channel = self.client.get_channel(self.channel_name)
        channel_event_hub = channel.newChannelEventHub(peer, org_admin)
        stream = channel_event_hub.connect(start='oldest',
                                           stop='newest', filtered=False)

        self.txs = {}
        channel_event_hub.registerTxEvent('all', onEvent=self.onTxEvent)

        try:
            await shield(stream)
        except Exception:
            pass

        channel_event_hub.disconnect()

        self.assertEqual(len(self.txs['all']), 4)

    def test_in_sequence(self):

        loop = asyncio.get_event_loop()

        logger.info("\n\nE2E testing started...")

        self.client.new_channel(SYSTEM_CHANNEL_NAME)

        loop.run_until_complete(self.get_channel_config_with_orderer())

        loop.run_until_complete(self.channel_create())

        loop.run_until_complete(self.channel_join())

        loop.run_until_complete(self.get_channel_config())

        loop.run_until_complete(self.chaincode_install())

        self.chaincode_install_fail()

        loop.run_until_complete(self.query_installed_chaincodes())

        loop.run_until_complete(self.chaincode_instantiate())

        loop.run_until_complete(self.query_instantiated_chaincodes())

        loop.run_until_complete(self.chaincode_invoke())

        loop.run_until_complete(self.chaincode_invoke_fail())

        loop.run_until_complete(self.chaincode_channel_event_hub())

        loop.run_until_complete(self.chaincode_query())

        loop.run_until_complete(self.query_channels())

        loop.run_until_complete(self.query_info())

        loop.run_until_complete(self.query_block_by_txid())

        loop.run_until_complete(self.query_block_by_hash())

        loop.run_until_complete(self.query_block())

        loop.run_until_complete(self.query_transaction())

        loop.run_until_complete(self.get_filtered_block_events())

        loop.run_until_complete(self.get_full_block_events())

        loop.run_until_complete(self.get_tx_events())

        logger.info("E2E all test cases done\n\n")


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: Apache-2.0

import asyncio
import logging
from asyncio import sleep

from hfc.fabric import Client
from hfc.fabric.peer import create_peer
from test.integration.utils import BaseTestCase
from test.integration.config import E2E_CONFIG

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
test_network = E2E_CONFIG['test-network']
CC_PATH = 'github.com/example_cc'
CC_NAME = 'example_cc'
CC_VERSION = '1.0'


class DiscoveryTest(BaseTestCase):

    def test_discovery(self):
        loop = asyncio.get_event_loop()

        org1 = 'org1.example.com'
        peer_config = test_network['org1.example.com']['peers']['peer0']
        tls_cacerts = peer_config['tls_cacerts']
        opts = (('grpc.ssl_target_name_override',
                 peer_config['server_hostname']),)
        endpoint = peer_config['grpc_request_endpoint']

        peer = create_peer(endpoint=endpoint,
                           tls_cacerts=tls_cacerts,
                           opts=opts)

        # Channel create
        response = loop.run_until_complete(
            self.client.channel_create('orderer.example.com',
                                       self.channel_name,
                                       self.user,
                                       config_yaml=self.config_yaml,
                                       channel_profile=self.channel_profile))
        logger.debug(response)
        self.assertTrue(response)

        # Channel join
        channel = self.client.get_channel(self.channel_name)
        self.assertIsNotNone(channel)

        response = loop.run_until_complete(self.client.channel_join(
            requestor=self.user,
            channel_name=self.channel_name,
            peers=['peer0.' + org1, 'peer1.' + org1],
            orderer='orderer.example.com'
        ))
        self.assertTrue(response)

        # CC install
        responses = loop.run_until_complete(self.client.chaincode_install(
            requestor=self.user,
            peers=['peer0.' + org1, 'peer1.' + org1],
            cc_path=CC_PATH,
            cc_name=CC_NAME,
            cc_version=CC_VERSION
        ))
        self.assertTrue(responses)

        # CC instantiate
        args = ['a', '200', 'b', '300']
        response = loop.run_until_complete(self.client.chaincode_instantiate(
            requestor=self.user,
            channel_name=self.channel_name,
            peers=['peer0.' + org1],
            args=args,
            cc_name=CC_NAME,
            cc_version=CC_VERSION,
            wait_for_event=True
        ))
        logger.debug(response)
        self.assertTrue(response)

        # Query instantiated cc
        responses = loop.run_until_complete(
            self.client.query_instantiated_chaincodes(
                requestor=self.user,
                channel_name=self.channel_name,
                peers=['peer0.' + org1, 'peer1.' + org1]
            ))
        logger.debug(responses)
        self.assertEqual(len(responses), 2)
        '''[chaincodes {
           name: "example_cc"
           version: "1.0"
           path: "github.com/example_cc"
           input: "<nil>"
           escc: "escc"
           vscc: "vscc"
        }
        , chaincodes {
           name: "example_cc"
           version: "1.0"
           path: "github.com/example_cc"
           input: "<nil>"
           escc: "escc"
           vscc: "vscc"
        }
        ]'''

        # sleep 3 seconds for having all peers for discovery
        loop.run_until_complete(sleep(5))

        # TEST: config
        # this one contain 3 queries
        response = loop.run_until_complete(channel._discovery(
            requestor=self.user,
            target=peer,
            config=True,
            interests=[{'chaincodes': [{'name': CC_NAME}]}]
        ))

        self.assertEqual(
            response.results[1].config_result.msps['OrdererMSP'].name,
            'OrdererMSP')
        self.assertEqual(
            list(response.results[0].members.peers_by_org.keys())[0],
            'Org1MSP')
        self.assertEqual(
            response.results[2].cc_query_res.content[0].chaincode, CC_NAME)

        # TEST: query_peer
        results = loop.run_until_complete(
            self.client.query_peers(self.user, peer))
        self.assertEqual(len(results['local_peers']['Org1MSP']['peers']), 2)
        self.assertEqual(results['local_peers']['Org1MSP']['peers']
                         [0]['endpoint'], 'peer0.org1.example.com:7051')
        self.assertEqual(results['local_peers']['Org1MSP']['peers']
                         [1]['endpoint'], 'peer1.org1.example.com:7051')

        # Test init with discovery

        client_discovery = Client()

        loop.run_until_complete(
            client_discovery.init_with_discovery(self.user, peer,
                                                 self.channel_name))

        self.assertEqual(len(client_discovery._orderers), 1)
        self.assertEqual(len(client_discovery._peers), 3)
        self.assertEqual(len(client_discovery._organizations), 3)



# Copyright IBM Corp. 2016 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import unittest

from cryptography import x509
from cryptography.hazmat.primitives.serialization import Encoding
from cryptography.x509 import NameOID

from hfc.util.crypto.crypto import \
    CURVE_P_256_Size, SHA3, SHA2, CURVE_P_384_Size, ecies


class CryptoTest(unittest.TestCase):
    """Test for crypto module. """

    def setUp(self):
        self.plain_text = b'Hello world!'

    def test_ecies_secp384r1_sha3(self):
        """Test case for security level 384, hash SHA3."""
        ecies384 = ecies(CURVE_P_384_Size, SHA3)
        private_key = ecies384.generate_private_key()
        cipher_text = ecies384.encrypt(private_key.public_key(),
                                       self.plain_text)

        self.assertEqual(ecies384.decrypt(private_key, cipher_text),
                         self.plain_text)

    def test_ecies_secp256r1_sha3(self):
        """Test case for security level 256, hash SHA3."""
        ecies256 = ecies(CURVE_P_256_Size, SHA3)
        private_key = ecies256.generate_private_key()
        cipher_text = ecies256.encrypt(private_key.public_key(),
                                       self.plain_text)

        self.assertEqual(ecies256.decrypt(private_key, cipher_text),
                         self.plain_text)

    def test_ecies_secp256r1_sha2(self):
        """Test case for security level 256, hash SHA2."""
        ecies256 = ecies(CURVE_P_256_Size, SHA2)
        private_key = ecies256.generate_private_key()
        cipher_text = ecies256.encrypt(private_key.public_key(),
                                       self.plain_text)

        self.assertEqual(ecies256.decrypt(private_key, cipher_text),
                         self.plain_text)

    def test_ecies_secp256r1_sha2_sign_verify(self):
        """Test case for security level 256, hash SHA2."""
        ecies256 = ecies(CURVE_P_256_Size, SHA2)
        private_key = ecies256.generate_private_key()
        signature = ecies256.sign(private_key,
                                  self.plain_text)

        self.assertEqual(ecies256.verify(private_key.public_key(),
                                         self.plain_text,
                                         signature),
                         True)

        self.assertEqual(ecies256.verify(private_key.public_key(),
                                         self.plain_text + b'!',
                                         signature),
                         False)

    def test_ecies_secp384r1_sha2_sign_verify(self):
        """Test case for security level 256, hash SHA2."""
        ecies384 = ecies(CURVE_P_384_Size, SHA2)
        private_key = ecies384.generate_private_key()
        signature = ecies384.sign(private_key,
                                  self.plain_text)

        self.assertEqual(ecies384.verify(private_key.public_key(),
                                         self.plain_text,
                                         signature),
                         True)

        self.assertEqual(ecies384.verify(private_key.public_key(),
                                         self.plain_text + b'!',
                                         signature),
                         False)

    def test_ecies_generate_csr(self):
        """Test case for generate certificate signing request."""
        ecies256 = ecies()
        private_key = ecies256.generate_private_key()
        csr = ecies256.generate_csr(private_key, x509.Name(
            [x509.NameAttribute(NameOID.COMMON_NAME, u"test")]))
        csr_pem = csr.public_bytes(Encoding.PEM)
        self.assertTrue(csr_pem.startswith(
            b"-----BEGIN CERTIFICATE REQUEST-----"))

    def test_secp256r1_check_malleability_and_check_prevent_malleability(self):
        """Test case for check_malleability and prevent_malleability."""
        ecies256 = ecies(CURVE_P_256_Size, SHA2)
        private_key = ecies256.generate_private_key()
        signature = ecies256.sign(private_key,
                                  self.plain_text)

        self.assertEqual(ecies256._check_malleability(signature),
                         True)

        self.assertEqual(ecies256._check_malleability(
            ecies256._prevent_malleability(signature)),
            True)

    def test_secp384r1_check_malleability_and_check_prevent_malleability(self):
        """Test case for check_malleability and prevent_malleability."""
        ecies384 = ecies(CURVE_P_384_Size, SHA2)
        private_key = ecies384.generate_private_key()
        signature = ecies384.sign(private_key,
                                  self.plain_text)

        self.assertEqual(ecies384._check_malleability(signature),
                         True)

        self.assertEqual(ecies384._check_malleability(
            ecies384._prevent_malleability(signature)),
            True)


if __name__ == '__main__':
    unittest.main()



# Copyright Sudheesh Singanamalla 2018 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os
import unittest

from hfc.fabric.block_decoder import BlockDecoder
from hfc.fabric.block_decoder import HeaderType
from hfc.fabric.block_decoder import decode_readwrite_sets

# Import required Ledger Protos
from hfc.protos.ledger.rwset import rwset_pb2
from hfc.protos.ledger.rwset.kvrwset import kv_rwset_pb2


# Load a genesis block as input to test the decoders
with open(os.path.join(os.path.dirname(__file__),
                       "../fixtures/e2e_cli/" +
                       "channel-artifacts/orderer.genesis.block"),
          'rb') as f:
    data = f.read()

with open(os.path.join(os.path.dirname(__file__),
                       "../fixtures/e2e_cli/" +
                       "channel-artifacts/channel.tx"),
          'rb') as f:
    tx_data = f.read()

with open(os.path.join(os.path.dirname(__file__),
                       "../fixtures/e2e_cli/" +
                       "channel-artifacts/businesschannel_4.block"),
          'rb') as f:
    metadata_block = f.read()


class BlockDecoderTest(unittest.TestCase):
    """Test for BlockDecoder in Fabric"""

    def setUp(self):
        self._data = data
        self._metadata_block = metadata_block
        self.decoder_instance = BlockDecoder.decode(self._data)
        self._tx_data = tx_data
        self.decode_transaction = \
            BlockDecoder.decode_transaction(self._tx_data)
        self.decoded_metadata = BlockDecoder.decode(self._metadata_block)

    def test_decode_failure(self):
        """
        Checks failure cases for decode of block.
        """
        self.assertRaises(TypeError, lambda: BlockDecoder.decode())
        self.assertRaises(ValueError, lambda: BlockDecoder.decode('test'))

    def test_decode_block_header(self):
        """
        Checks if the block header has been decoded correctly.
        """
        data_hash = \
            b'e6a8b7ca7bf9aa5123e2d72bd3d55e0e32d6cefdf001bf31944061cef24c9ad3'
        previous_hash = b''
        number = 0
        header_info = self.decoder_instance['header']
        self.assertEqual(header_info['data_hash'], data_hash)
        self.assertEqual(header_info['previous_hash'], previous_hash)
        self.assertEqual(header_info['number'], number)

    def test_decode_block_data(self):
        """
        Checks if the block data has been decoded correctly.
        This test case also verifies the results from other
        decode header and data calls while decoding the block.
        """
        sh_nonce = b'71fba48e569c9f9b3f393e5b4b803db6209cb5fb2956301e'
        channel_id = 'testchainid'
        timestamp = '2018-10-19 03:53:04'
        tx_id = \
            '094760b6a9c9fdbb5d231c3ca79fe28ba2fd62a1aa9d913d37f7938ac44ba52a'
        epoch = 0
        version = 1
        type_value = 1
        extension = b''

        data_info = self.decoder_instance['data']
        data_item = data_info['data'][0]
        data_payload_header = data_item['payload']['header']
        sh = data_payload_header['signature_header']
        ch = data_payload_header['channel_header']

        self.assertEqual(len(data_info), 1)
        self.assertEqual(len(data_info['data']), 1)
        self.assertEqual(len(data_item.keys()), 2)
        self.assertEqual(len(data_payload_header.keys()), 2)
        self.assertEqual(len(sh['creator'].keys()), 2)
        self.assertEqual(len(sh.keys()), 2)
        self.assertEqual(len(ch.keys()), 8)

        self.assertEqual(sh['nonce'], sh_nonce)
        self.assertEqual(ch['extension'], extension)
        self.assertEqual(ch['channel_id'], channel_id)
        self.assertEqual(ch['timestamp'], timestamp)
        self.assertEqual(ch['tx_id'], tx_id)
        self.assertEqual(ch['epoch'], epoch)
        self.assertEqual(ch['version'], version)
        self.assertEqual(ch['type'], type_value)

    def test_decode_transaction(self):
        """
        Checks if a transaction has been decoded correctly.
        """
        transaction_envelope = self.decode_transaction['transaction_envelope']
        validation_code = self.decode_transaction['validation_code']
        tx_payload = transaction_envelope['payload']
        tx_payload_header = tx_payload['header']
        tx_channel_header = tx_payload_header['channel_header']
        tx_signature_header = tx_payload_header['signature_header']
        self.assertEqual(type(dict()), type(transaction_envelope))
        self.assertEqual(validation_code, 0)
        self.assertIn('header', tx_payload)
        self.assertIn('channel_header', tx_payload_header)
        self.assertIn('signature_header', tx_payload_header)
        self.assertIn('channel_id', tx_channel_header)
        self.assertIn('epoch', tx_channel_header)
        self.assertIn('extension', tx_channel_header)
        self.assertIn('timestamp', tx_channel_header)
        self.assertIn('tx_id', tx_channel_header)
        self.assertIn('type', tx_channel_header)
        self.assertIn('version', tx_channel_header)
        self.assertIn('creator', tx_signature_header)
        self.assertIn('nonce', tx_signature_header)

    def test_decode_block_metadata(self):
        """
        Checks if the metadata for the block has been decoded properly.
        """
        index = 2

        decoded_metadata = self.decoded_metadata
        block_metadata = decoded_metadata['metadata']
        metadata = block_metadata['metadata']
        sample_signature = metadata[1]
        self.assertIn('metadata', decoded_metadata)
        self.assertIn('metadata', block_metadata)
        self.assertIn('value', metadata[0])
        self.assertIn('signatures', metadata[0])
        self.assertIn('signature_header', sample_signature['signatures'][0])
        self.assertIn('signature', sample_signature['signatures'][0])
        self.assertEqual(sample_signature['value']['index'], index)
        self.assertEqual(len(metadata), 3)
        self.assertEqual(len(sample_signature['signatures']), 1)

    def test_decode_read_write_sets(self):
        """
        checks if the decode read write sets have been decoded properly.
        """
        kv_read_proto = kv_rwset_pb2.KVRead()
        version_proto = kv_rwset_pb2.Version()
        version_proto.block_num = 12
        version_proto.tx_num = 21
        kv_read_proto.version.CopyFrom(version_proto)
        kv_read_proto.key = 'read key'
        reads_array = []
        reads_array.append(kv_read_proto)

        range_query_info_proto = kv_rwset_pb2.RangeQueryInfo()
        range_query_info_proto.start_key = 'start'
        range_query_info_proto.end_key = 'end'
        range_query_info_proto.itr_exhausted = False
        range_query_info_array = []
        range_query_info_array.append(range_query_info_proto)

        kv_write_proto = kv_rwset_pb2.KVWrite()
        kv_write_proto.key = 'write key'
        kv_write_proto.is_delete = False
        kv_write_proto.value = b'this is the value'
        writes_array = []
        writes_array.append(kv_write_proto)

        kvrwset_proto = kv_rwset_pb2.KVRWSet()
        kvrwset_proto.reads.extend(reads_array)
        kvrwset_proto.range_queries_info.extend(range_query_info_array)
        kvrwset_proto.writes.extend(writes_array)

        results_proto = rwset_pb2.TxReadWriteSet()
        results_proto.data_model = rwset_pb2.TxReadWriteSet.KV
        ns_rwset_array = []
        ns_rwset_proto = rwset_pb2.NsReadWriteSet()
        ns_rwset_proto.namespace = 'testnamespace'
        ns_rwset_proto.rwset = kvrwset_proto.SerializeToString()
        ns_rwset_array.append(ns_rwset_proto)
        results_proto.ns_rwset.extend(ns_rwset_array)

        results_json = decode_readwrite_sets(results_proto.SerializeToString())
        nsrwset_value = results_json['ns_rwset'][0]
        nsrwset_reads = nsrwset_value['rwset']['reads']
        nsrwset_range_queries = nsrwset_value['rwset']['range_queries_info'][0]

        self.assertEqual('testnamespace', nsrwset_value['namespace'])
        self.assertEqual('read key', nsrwset_reads[0]['key'])
        self.assertEqual('12', nsrwset_reads[0]['version']['block_num'])
        self.assertEqual('21', nsrwset_reads[0]['version']['tx_num'])

        self.assertIn('range_queries_info', nsrwset_value['rwset'])
        self.assertEqual('end', nsrwset_range_queries['end_key'])
        self.assertEqual('start', nsrwset_range_queries['start_key'])
        self.assertIn('reads_merkle_hashes', nsrwset_range_queries)
        self.assertIn('raw_reads', nsrwset_range_queries)

        self.assertIn('writes', nsrwset_value['rwset'])
        self.assertEqual('write key',
                         nsrwset_value['rwset']['writes'][0]['key'])
        self.assertEqual(b'this is the value',
                         nsrwset_value['rwset']['writes'][0]['value'])

    def test_decode_header_type(self):
        """
        Test cases for decoding headertype
        """
        self.assertEqual('MESSAGE', HeaderType.convert_to_string(0))
        self.assertEqual('CONFIG', HeaderType.convert_to_string(1))
        self.assertEqual('CONFIG_UPDATE', HeaderType.convert_to_string(2))
        self.assertEqual('ENDORSER_TRANSACTION',
                         HeaderType.convert_to_string(3))
        self.assertEqual('ORDERER_TRANSACTION',
                         HeaderType.convert_to_string(4))
        self.assertEqual('DELIVER_SEEK_INFO', HeaderType.convert_to_string(5))
        self.assertEqual('CHAINCODE_PACKAGE', HeaderType.convert_to_string(6))
        self.assertEqual('UNKNOWN_TYPE', HeaderType.convert_to_string(99))


if __name__ == '__main__':
    unittest.main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/orderer/kafka.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/orderer/kafka.proto',
  package='orderer',
  syntax='proto3',
  serialized_options=_b('\n%org.hyperledger.fabric.protos.ordererZ,github.com/hyperledger/fabric/protos/orderer'),
  serialized_pb=_b('\n\x1ehfc/protos/orderer/kafka.proto\x12\x07orderer\"\xaf\x01\n\x0cKafkaMessage\x12/\n\x07regular\x18\x01 \x01(\x0b\x32\x1c.orderer.KafkaMessageRegularH\x00\x12\x35\n\x0btime_to_cut\x18\x02 \x01(\x0b\x32\x1e.orderer.KafkaMessageTimeToCutH\x00\x12/\n\x07\x63onnect\x18\x03 \x01(\x0b\x32\x1c.orderer.KafkaMessageConnectH\x00\x42\x06\n\x04Type\"\xb4\x01\n\x13KafkaMessageRegular\x12\x0f\n\x07payload\x18\x01 \x01(\x0c\x12\x12\n\nconfig_seq\x18\x02 \x01(\x04\x12\x31\n\x05\x63lass\x18\x03 \x01(\x0e\x32\".orderer.KafkaMessageRegular.Class\x12\x17\n\x0foriginal_offset\x18\x04 \x01(\x03\",\n\x05\x43lass\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06NORMAL\x10\x01\x12\n\n\x06\x43ONFIG\x10\x02\"-\n\x15KafkaMessageTimeToCut\x12\x14\n\x0c\x62lock_number\x18\x01 \x01(\x04\"&\n\x13KafkaMessageConnect\x12\x0f\n\x07payload\x18\x01 \x01(\x0c\"~\n\rKafkaMetadata\x12\x1d\n\x15last_offset_persisted\x18\x01 \x01(\x03\x12&\n\x1elast_original_offset_processed\x18\x02 \x01(\x03\x12&\n\x1elast_resubmitted_config_offset\x18\x03 \x01(\x03\x42U\n%org.hyperledger.fabric.protos.ordererZ,github.com/hyperledger/fabric/protos/ordererb\x06proto3')
)



_KAFKAMESSAGEREGULAR_CLASS = _descriptor.EnumDescriptor(
  name='Class',
  full_name='orderer.KafkaMessageRegular.Class',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NORMAL', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CONFIG', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=358,
  serialized_end=402,
)
_sym_db.RegisterEnumDescriptor(_KAFKAMESSAGEREGULAR_CLASS)


_KAFKAMESSAGE = _descriptor.Descriptor(
  name='KafkaMessage',
  full_name='orderer.KafkaMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='regular', full_name='orderer.KafkaMessage.regular', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time_to_cut', full_name='orderer.KafkaMessage.time_to_cut', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='connect', full_name='orderer.KafkaMessage.connect', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Type', full_name='orderer.KafkaMessage.Type',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=44,
  serialized_end=219,
)


_KAFKAMESSAGEREGULAR = _descriptor.Descriptor(
  name='KafkaMessageRegular',
  full_name='orderer.KafkaMessageRegular',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='payload', full_name='orderer.KafkaMessageRegular.payload', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config_seq', full_name='orderer.KafkaMessageRegular.config_seq', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='class', full_name='orderer.KafkaMessageRegular.class', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='original_offset', full_name='orderer.KafkaMessageRegular.original_offset', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _KAFKAMESSAGEREGULAR_CLASS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=222,
  serialized_end=402,
)


_KAFKAMESSAGETIMETOCUT = _descriptor.Descriptor(
  name='KafkaMessageTimeToCut',
  full_name='orderer.KafkaMessageTimeToCut',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_number', full_name='orderer.KafkaMessageTimeToCut.block_number', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=404,
  serialized_end=449,
)


_KAFKAMESSAGECONNECT = _descriptor.Descriptor(
  name='KafkaMessageConnect',
  full_name='orderer.KafkaMessageConnect',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='payload', full_name='orderer.KafkaMessageConnect.payload', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=451,
  serialized_end=489,
)


_KAFKAMETADATA = _descriptor.Descriptor(
  name='KafkaMetadata',
  full_name='orderer.KafkaMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='last_offset_persisted', full_name='orderer.KafkaMetadata.last_offset_persisted', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_original_offset_processed', full_name='orderer.KafkaMetadata.last_original_offset_processed', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_resubmitted_config_offset', full_name='orderer.KafkaMetadata.last_resubmitted_config_offset', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=491,
  serialized_end=617,
)

_KAFKAMESSAGE.fields_by_name['regular'].message_type = _KAFKAMESSAGEREGULAR
_KAFKAMESSAGE.fields_by_name['time_to_cut'].message_type = _KAFKAMESSAGETIMETOCUT
_KAFKAMESSAGE.fields_by_name['connect'].message_type = _KAFKAMESSAGECONNECT
_KAFKAMESSAGE.oneofs_by_name['Type'].fields.append(
  _KAFKAMESSAGE.fields_by_name['regular'])
_KAFKAMESSAGE.fields_by_name['regular'].containing_oneof = _KAFKAMESSAGE.oneofs_by_name['Type']
_KAFKAMESSAGE.oneofs_by_name['Type'].fields.append(
  _KAFKAMESSAGE.fields_by_name['time_to_cut'])
_KAFKAMESSAGE.fields_by_name['time_to_cut'].containing_oneof = _KAFKAMESSAGE.oneofs_by_name['Type']
_KAFKAMESSAGE.oneofs_by_name['Type'].fields.append(
  _KAFKAMESSAGE.fields_by_name['connect'])
_KAFKAMESSAGE.fields_by_name['connect'].containing_oneof = _KAFKAMESSAGE.oneofs_by_name['Type']
_KAFKAMESSAGEREGULAR.fields_by_name['class'].enum_type = _KAFKAMESSAGEREGULAR_CLASS
_KAFKAMESSAGEREGULAR_CLASS.containing_type = _KAFKAMESSAGEREGULAR
DESCRIPTOR.message_types_by_name['KafkaMessage'] = _KAFKAMESSAGE
DESCRIPTOR.message_types_by_name['KafkaMessageRegular'] = _KAFKAMESSAGEREGULAR
DESCRIPTOR.message_types_by_name['KafkaMessageTimeToCut'] = _KAFKAMESSAGETIMETOCUT
DESCRIPTOR.message_types_by_name['KafkaMessageConnect'] = _KAFKAMESSAGECONNECT
DESCRIPTOR.message_types_by_name['KafkaMetadata'] = _KAFKAMETADATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

KafkaMessage = _reflection.GeneratedProtocolMessageType('KafkaMessage', (_message.Message,), dict(
  DESCRIPTOR = _KAFKAMESSAGE,
  __module__ = 'hfc.protos.orderer.kafka_pb2'
  # @@protoc_insertion_point(class_scope:orderer.KafkaMessage)
  ))
_sym_db.RegisterMessage(KafkaMessage)

KafkaMessageRegular = _reflection.GeneratedProtocolMessageType('KafkaMessageRegular', (_message.Message,), dict(
  DESCRIPTOR = _KAFKAMESSAGEREGULAR,
  __module__ = 'hfc.protos.orderer.kafka_pb2'
  # @@protoc_insertion_point(class_scope:orderer.KafkaMessageRegular)
  ))
_sym_db.RegisterMessage(KafkaMessageRegular)

KafkaMessageTimeToCut = _reflection.GeneratedProtocolMessageType('KafkaMessageTimeToCut', (_message.Message,), dict(
  DESCRIPTOR = _KAFKAMESSAGETIMETOCUT,
  __module__ = 'hfc.protos.orderer.kafka_pb2'
  # @@protoc_insertion_point(class_scope:orderer.KafkaMessageTimeToCut)
  ))
_sym_db.RegisterMessage(KafkaMessageTimeToCut)

KafkaMessageConnect = _reflection.GeneratedProtocolMessageType('KafkaMessageConnect', (_message.Message,), dict(
  DESCRIPTOR = _KAFKAMESSAGECONNECT,
  __module__ = 'hfc.protos.orderer.kafka_pb2'
  # @@protoc_insertion_point(class_scope:orderer.KafkaMessageConnect)
  ))
_sym_db.RegisterMessage(KafkaMessageConnect)

KafkaMetadata = _reflection.GeneratedProtocolMessageType('KafkaMetadata', (_message.Message,), dict(
  DESCRIPTOR = _KAFKAMETADATA,
  __module__ = 'hfc.protos.orderer.kafka_pb2'
  # @@protoc_insertion_point(class_scope:orderer.KafkaMetadata)
  ))
_sym_db.RegisterMessage(KafkaMetadata)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Copyright IBM Corp. 2016 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import aiogrpc


def create_grpc_channel(target, cert_file=None, client_key=None,
                        client_cert=None, opts=None):
    """Construct a grpc channel.

    :param target: server address include host:port
    :param cert_file: ssl/tls root cert file for the connection (Default value = None)
    :param opts: grpc channel options
                grpc.default_authority: default authority
                grpc.ssl_target_name_override: ssl target name override
    :param client_key: client key (Default value = None)
    :param client_cert: client certificate (Default value = None)
    :return: grpc channel
    """

    root_cert = None

    if cert_file:
        if isinstance(cert_file, bytes):
            root_cert = cert_file
        else:
            with open(cert_file, 'rb') as f:
                root_cert = f.read()

    if client_key:
        if not isinstance(client_key, bytes):
            with open(client_key, 'rb') as f:
                client_key = f.read()

    if client_cert:
        if not isinstance(client_cert, bytes):
            with open(client_cert, 'rb') as f:
                client_cert = f.read()

    if root_cert is None:
        return aiogrpc.insecure_channel(target, opts)
    else:
        if client_cert and client_key:
            creds = aiogrpc. \
                ssl_channel_credentials(root_cert,
                                        private_key=client_key,
                                        certificate_chain=client_cert)
        else:
            creds = aiogrpc.ssl_channel_credentials(root_cert)

        return aiogrpc.secure_channel(target, creds, opts)
